var fzami = null;

var fzamiLoad = function() {
  fzami = new FZami();

  Components.classes["@mozilla.org/observer-service;1"]
            .getService(Components.interfaces.nsIObserverService)
            .addObserver(fzami, "fzami:preferences-changed", false);

  fzami.start();
};

var fzamiUnload = function() {
  if (fzami != null)
  {
    Components.classes["@mozilla.org/observer-service;1"]
            .getService(Components.interfaces.nsIObserverService)
            .removeObserver(fzami, "fzami:preferences-changed");
    fzami.stop();
  }
  fzami = null;
}

var fzamiRefresh = function() {
  if (fzami != null)
  {
    fzami.reload();
  } else {
    fzamiLoad();
  }
}


function openLink(url)
{
  var browser = window.document.getElementById("content");
  browser.loadURI(url);
  window.content.focus();
}




function FZami() {};

FZami.prototype = {
  preferences: Components.classes["@mozilla.org/preferences-service;1"]
                                .getService(Components.interfaces.nsIPrefBranch),
  interval: null,
  prayertimes: null,
  currentPrayer: null,
  dateOfPrayerTimes: null,
  server: null,
  errorMessage: null,
  endPrayerAlertShown: false,

  start : function() {
      this.reset();
      this.server = new IslamicFinderServer(this);
      setTimeout("if (fzami != null) fzami.reload();", 100);
      this.interval = setInterval("if (fzami != null) fzami.rerender();", 60000);
  },

  stop : function() {
      clearInterval(this.interval);
      this.interval = null;
  },

  reload : function() {
      this.showProgress();
      this.server.reloadPrayerTimes();
  },

  reset : function() {
      this.prayertimes = null;
      this.dateOfPrayerTimes = null;
      this.currentPrayer = null;
      this.errorMessage = null;
      this.endPrayerAlertShown = false;
  },

  observe : function(subject, topic, data) {
	if (topic == "fzami:preferences-changed") {
		setTimeout("if (fzami != null) fzami.reload();", 100);
	}
  },

  showError : function(errMsg) {
        this.reset();
        this.errorMessage = errMsg;
	var statusElement = document.getElementById("fzami-panel");
	statusElement.setAttribute("status", "enabled");
	var errorDescElement = document.getElementById("fzami-errormsg");
	errorDescElement.setAttribute("value", errMsg);
	var deck = document.getElementById("fzami-deck");

        this.setToolTip();
	for (var i = 0; i < deck.childNodes.length; i++) deck.childNodes[i].setAttribute("collapsed", "true");
	document.getElementById("fzami-error-box").setAttribute("collapsed", "false");
  },

  showSuccess : function() {
	var statusElement = document.getElementById("fzami-panel");
	statusElement.setAttribute("status", "enabled");
	var deck = document.getElementById("fzami-deck");
	for (var i = 0; i < deck.childNodes.length; i++) deck.childNodes[i].setAttribute("collapsed", "true");
	var displayType = this.preferences.getIntPref("extensions.fzami.displayType");

	var boxName = null;
	switch(displayType)
	{
		case 0 : boxName = "fzami-one-prayer-box"; break;
		case 1 : boxName = "fzami-all-prayers-box"; break;
		case 2 :
		default : boxName = "fzami-text-box"; break;
	}
	document.getElementById(boxName).setAttribute("collapsed", "false");
  },

  showProgress : function() {
	var deck = document.getElementById("fzami-deck");
	for (var i = 0; i < deck.childNodes.length; i++) deck.childNodes[i].setAttribute("collapsed", "true");
	document.getElementById("fzami-progressmeter").setAttribute("collapsed", "false");
  },

  setPrayerTimes : function(prayertimes) {
        this.reset();
	this.prayertimes = prayertimes;
        this.dateOfPrayerTimes = new Date();
	this.rerender();
  },

  rerender: function()
  {
    if (this.prayertimes != null) {
      this.recalculateCurrentPrayer();
      this.redisplayPrayerTimes();
      this.setToolTip();
      this.highlightCurrentPrayer();
      this.showSuccess();
    }
  },

  setToolTip : function() {
	var tooltip = document.getElementById("fzami-tooltip");
        while (tooltip.childNodes.length > 0) tooltip.removeChild(tooltip.childNodes[0]);
        tooltip.removeAttribute("title");
        tooltip.removeAttribute("label");
        if (this.errorMessage != null)
        {
            var vbox = document.createElement("vbox");
		tooltip.appendChild(vbox);

		var description = document.createElement("description");
		description.setAttribute("class", "fzami-tooltip-title");
		description.setAttribute("value", "Error");
		vbox.appendChild(description);

		description = document.createElement("description");
		vbox.appendChild(description);
		description.setAttribute("value", "An error has occurred.  This usually means that you");

		description = document.createElement("description");
		vbox.appendChild(description);
		description.setAttribute("value", "do not have a live internet connection, but it may");

		description = document.createElement("description");
		vbox.appendChild(description);
		description.setAttribute("value", "also be caused by invalid option settings or a bug");

		description = document.createElement("description");
		vbox.appendChild(description);
		description.setAttribute("value", "in fzami.  If you feel that this is a bug, please");

		description = document.createElement("description");
		vbox.appendChild(description);
		description.setAttribute("value", "report it at http://fzami.mozdev.org/.");
        } else if (this.prayertimes != null) {

      	var location = "unknown";
		if (this.preferences.getIntPref("extensions.fzami.location.type") == 0)
		{
			location = this.preferences.getCharPref("extensions.fzami.location.zipcode");
		} else {
			location = this.preferences.getCharPref("extensions.fzami.location.city");
		}
            var date = (this.dateOfPrayerTimes.getYear()+1900)+"-"+(this.dateOfPrayerTimes.getMonth()+1)+"-"+this.dateOfPrayerTimes.getDate();
	var militaryTime = this.preferences.getBoolPref("extensions.fzami.militaryTime");

            var vbox = document.createElement("vbox");
		tooltip.appendChild(vbox);

		var description = document.createElement("description");
		description.setAttribute("class", "fzami-tooltip-title");
		description.setAttribute("value","Prayer Times");
		vbox.appendChild(description);

		var separator = document.createElement("separator");
		separator.setAttribute("class", "groove-thin");
		vbox.appendChild(separator);

		description = document.createElement("description");
		description.setAttribute("value", "Location: "+location);
		vbox.appendChild(description);

		description = document.createElement("description");
		description.setAttribute("value", "Date: "+date);
		vbox.appendChild(description);

		separator = document.createElement("separator");
		separator.setAttribute("class", "groove-thin");
		vbox.appendChild(separator);

		var grid = document.createElement("grid");
		vbox.appendChild(grid);
		var rows = document.createElement("rows");
		grid.appendChild(rows);

		var row = document.createElement("row");
		row.setAttribute("id", "fzami-tooltip-fajr-time-row");
		rows.appendChild(row);
		var prayerName = document.createElement("label");
		prayerName.setAttribute("value", "fajr");
		row.appendChild(prayerName);
		var prayerTime = document.createElement("label");
		prayerTime.setAttribute("value", this.prayertimes["fajr"].toString(militaryTime));
		row.appendChild(prayerTime);

		row = document.createElement("row");
		row.setAttribute("id", "fzami-tooltip-shuruk-time-row");
		rows.appendChild(row);
		prayerName = document.createElement("label");
		prayerName.setAttribute("value", "shuruk");
		row.appendChild(prayerName);
		prayerTime = document.createElement("label");
		prayerTime.setAttribute("value", this.prayertimes["shuruk"].toString(militaryTime));
		row.appendChild(prayerTime);

		row = document.createElement("row");
		row.setAttribute("id", "fzami-tooltip-zuhr-time-row");
		rows.appendChild(row);
		prayerName = document.createElement("label");
		prayerName.setAttribute("value", "zuhr");
		row.appendChild(prayerName);
		prayerTime = document.createElement("label");
		prayerTime.setAttribute("value", this.prayertimes["zuhr"].toString(militaryTime));
		row.appendChild(prayerTime);

		row = document.createElement("row");
		row.setAttribute("id", "fzami-tooltip-asr-time-row");
		rows.appendChild(row);
		prayerName = document.createElement("label");
		prayerName.setAttribute("value", "asr");
		row.appendChild(prayerName);
		prayerTime = document.createElement("label");
		prayerTime.setAttribute("value", this.prayertimes["asr"].toString(militaryTime));
		row.appendChild(prayerTime);

		row = document.createElement("row");
		row.setAttribute("id", "fzami-tooltip-maghrib-time-row");
		rows.appendChild(row);
		prayerName = document.createElement("label");
		prayerName.setAttribute("value", "maghrib");
		row.appendChild(prayerName);
		prayerTime = document.createElement("label");
		prayerTime.setAttribute("value", this.prayertimes["maghrib"].toString(militaryTime));
		row.appendChild(prayerTime);

		row = document.createElement("row");
		row.setAttribute("id", "fzami-tooltip-isha-time-row");
		rows.appendChild(row);
		prayerName = document.createElement("label");
		prayerName.setAttribute("value", "isha");
		row.appendChild(prayerName);
		prayerTime = document.createElement("label");
		prayerTime.setAttribute("value", this.prayertimes["isha"].toString(militaryTime));
		row.appendChild(prayerTime);
        }
  },

  redisplayPrayerTimes : function() {
	if (this.prayertimes == null) return;
	var militaryTime = this.preferences.getBoolPref("extensions.fzami.militaryTime");

	document.getElementById("fzami-fajr-time").value = this.prayertimes['fajr'].toString(militaryTime)+" - "+this.prayertimes['shuruk'].toString(militaryTime);
	document.getElementById("fzami-zuhr-time").value = this.prayertimes['zuhr'].toString(militaryTime);
	document.getElementById("fzami-asr-time").value = this.prayertimes['asr'].toString(militaryTime);
	document.getElementById("fzami-maghrib-time").value = this.prayertimes['maghrib'].toString(militaryTime);
	document.getElementById("fzami-isha-time").value = this.prayertimes['isha'].toString(militaryTime);

	if (this.currentPrayer != null && this.currentPrayer != 'shuruk')
	{
		document.getElementById("fzami-current-prayer-label").value = this.currentPrayer+":";
		document.getElementById("fzami-current-prayer-time").value = this.prayertimes[this.currentPrayer].toString(militaryTime)+" - "+this.getNextPrayerTime().toString(militaryTime);
	} else if (this.currentPrayer == 'shuruk') {
		document.getElementById("fzami-current-prayer-label").value = "";
		document.getElementById("fzami-current-prayer-time").value = "zuhr starts at "+this.prayertimes['zuhr'].toString(militaryTime);
	} else {
		document.getElementById("fzami-current-prayer-label").value = "";
		document.getElementById("fzami-current-prayer-time").value = "no current prayer time";
	}
  },


  recalculateCurrentPrayer: function()
  {
	var oldPrayerName = this.currentPrayer;
	this.currentPrayer = null;
	if (this.prayertimes == null) return;
	var currentTime = new Date();

	if (this.prayertimes['fajr'].before(currentTime) && this.prayertimes['shuruk'].after(currentTime)) this.currentPrayer = 'fajr';
	if (this.prayertimes['shuruk'].before(currentTime) && this.prayertimes['zuhr'].after(currentTime)) this.currentPrayer = 'shuruk';
	if (this.prayertimes['zuhr'].before(currentTime) && this.prayertimes['asr'].after(currentTime)) this.currentPrayer = 'zuhr';
	if (this.prayertimes['asr'].before(currentTime) && this.prayertimes['maghrib'].after(currentTime)) this.currentPrayer = 'asr';
	if (this.prayertimes['maghrib'].before(currentTime) && this.prayertimes['isha'].after(currentTime)) this.currentPrayer = 'maghrib';
	if (this.prayertimes['isha'].before(currentTime) || this.prayertimes['fajr'].after(currentTime)) this.currentPrayer = 'isha';

        if (oldPrayerName != null && this.currentPrayer != 'shuruk' && this.currentPrayer != oldPrayerName)
	{
		this.endPrayerAlertShown = false;
		this.showStartPrayerTimeAlert();
	} else if (this.currentPrayer != null && this.currentPrayer != 'shuruk' && !this.endPrayerAlertShown) {
		if (this.isInEndingAlertTimeSpan())
		{
			this.showEndPrayerTimeAlert();
		}
	}
  },

  isInEndingAlertTimeSpan: function()
  {
	var now = new Date();
	var nextPrayerTime = this.getNextPrayerTime();
	if (nextPrayerTime != null && nextPrayerTime.after(now))
	{
		var endBuffer = this.preferences.getIntPref("extensions.fzami.alert.end.buffer");
		now.setMinutes(now.getMinutes()+endBuffer);
		if (nextPrayerTime.before(now))
		{
			return true;
		}
	}
	return false;
  },

  getNextPrayerName: function()
  {
	switch (this.currentPrayer)
	{
		case 'fajr' : return 'shuruk'; break;
		case 'shuruk' : return 'zuhr'; break;
		case 'zuhr' : return 'asr'; break;
		case 'asr' : return 'maghrib'; break;
		case 'maghrib' : return 'isha'; break;
		case 'isha' : return 'fajr'; break;
		default : return null;
	}
	return null;
  },

  getNextPrayerTime: function()
  {
	var prayer = this.getNextPrayerName();
	if (prayer == null || this.prayertimes == null) return null;
	return this.prayertimes[prayer];
  },

  highlightCurrentPrayer: function()
  {
	document.getElementById("fzami-fajr-box").setAttribute("class", "fzami-prayer-time");
	document.getElementById("fzami-zuhr-box").setAttribute("class", "fzami-prayer-time");
	document.getElementById("fzami-asr-box").setAttribute("class", "fzami-prayer-time");
	document.getElementById("fzami-maghrib-box").setAttribute("class", "fzami-prayer-time");
	document.getElementById("fzami-isha-box").setAttribute("class", "fzami-prayer-time");
	document.getElementById("fzami-current-prayer-box").setAttribute("class", "fzami-prayer-time");
	document.getElementById("fzami-text-box").setAttribute("class", "fzami-box");

	document.getElementById("fzami-tooltip-fajr-time-row").setAttribute("class", "fzami-prayer-time");
	document.getElementById("fzami-tooltip-shuruk-time-row").setAttribute("class", "fzami-prayer-time");
	document.getElementById("fzami-tooltip-zuhr-time-row").setAttribute("class", "fzami-prayer-time");
	document.getElementById("fzami-tooltip-asr-time-row").setAttribute("class", "fzami-prayer-time");
	document.getElementById("fzami-tooltip-maghrib-time-row").setAttribute("class", "fzami-prayer-time");
	document.getElementById("fzami-tooltip-isha-time-row").setAttribute("class", "fzami-prayer-time");

	var classname = "fzami-current-prayer-time";
	if (this.isInEndingAlertTimeSpan())
	{
		classname = "fzami-current-prayer-time-ending";
		document.getElementById("fzami-current-prayer-box").setAttribute("class", "fzami-current-prayer-time-ending");
		document.getElementById("fzami-text-box").setAttribute("class", "fzami-current-prayer-time-ending");
	}

	switch (this.currentPrayer)
	{
		case 'fajr':
		case 'zuhr':
		case 'asr':
		case 'maghrib':
		case 'isha':
			document.getElementById("fzami-"+this.currentPrayer+"-box").setAttribute("class", classname);
			document.getElementById("fzami-tooltip-"+this.currentPrayer+"-time-row").setAttribute("class", classname);
			break;
	}
  },

  showStartPrayerTimeAlert : function()
  {
	var showStartAlert = this.preferences.getBoolPref("extensions.fzami.alert.start");
	if (showStartAlert)
	{
		var alerts = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
		alerts.showAlertNotification(null, 'fzami alert!', 'its '+this.currentPrayer+' time', false, null, this);
	}
  },

  showEndPrayerTimeAlert : function()
  {
	if (this.currentPrayer != null)
	{
		this.endPrayerAlertShown = true;
		var showEndAlert = this.preferences.getBoolPref("extensions.fzami.alert.end");
		var endBuffer = this.preferences.getIntPref("extensions.fzami.alert.end.buffer");
		if (showEndAlert)
		{
			var alerts = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
			alerts.showAlertNotification(null, 'fzami alert!', 'Less than '+endBuffer+' minutes left for '+this.currentPrayer+' time.', false, null, this);
		}
	}
	
  },
};


function trim(sInString) {
  sInString = sInString.replace( /^\s+/g, "" ); // strip leading
  return sInString.replace( /\s+$/g, "" ); // strip trailing
}

  